# make style string (or NULL) from option list
#
# INPUT:
#   lst: named list of option values
#   lineNumOffset: line number offset in input code for use in error reporting
# OUTPUT:
#   NULL or a CSS style string
# DETAILS:
#   The option string can set styles through specific arguments textColor=, backgroundColor=,
#   and/or borderColor=, as well as style=.  The latter looks like a CSS style
#   string, but must use "|" instead of ";" to separate elements, since ";" separates 
#   options in shinyTex itself.

makeStyle = function(lst, lineNumOffset) {
  # separate 'style=' from other style components
  style = lst[["style"]]
  stylePos = match(c("textColor", "backgroundColor", "borderColor"),
                   names(lst)) 
  stylePos = stylePos[!sapply(stylePos, is.na)]
  lst = lst[stylePos]

  # return NULL if no style elements are specified
  if (is.null(style) && length(lst)==0) return(NULL)

  # break up 'style' argument into its elements in named list format
  if (is.null(style)) {
    rtn = NULL
  } else {
    styleX = str_trim(strsplit(style, "[|]")[[1]])
    styleX = strsplit(styleX, ":")
    if (any(sapply(styleX, length) != 2)) {
      stop("@", lineNumOffset, " bad 'style=' (", style, ")")
    }
    rtn = lapply(styleX, function(x) x[2])
    names(rtn) = sapply(styleX, function(x) x[1])
  }

  # Add in explicit colors (giving error for ones already in 'style')
  if (!is.null(lst[["textColor"]])) {
    if (any(names(rtn) == "color")) {
      stop("@", lineNumOffset, " color specified twice")
    }
    rtn[["color"]] = lst[["textColor"]]
  }
  if (!is.null(lst[["backgroundColor"]])) {
    if (any(names(rtn) == "background-color")) {
      stop("@", lineNumOffset, " background-color specified twice")
    }
    rtn[["background-color"]] = lst[["backgroundColor"]]
  }
  if (!is.null(lst[["borderColor"]])) {
    if (any(names(rtn) == "border-color")) {
      stop("@", lineNumOffset, " border-color specified twice")
    }
    rtn[["border-color"]] = lst[["borderColor"]]
  }

  rtn = paste(sapply(1:length(rtn), 
                     function(n) paste0(names(rtn)[n], ':',
                                        as.character(rtn[n]))),
                                        collapse=";")
  return(rtn)
}
