# Description: extend and validate a list of options
# Input:
#   type: type of command being processed (for error reporting only)
#   options: a named list comprising the options
#   required: a character vector containing the names of the required arguments
#   requiredTypes: a character vector of types ('c','n','l', 'cv') matching 'required'
#   optional: a character vector containing the names of the optional arguments
#   optionalTypes: a character vector of types ('c','n','l') matching 'optional'
#   defaults: a named list of defaults for some of the optional elements
#
# Value: a named list containing all required options, supplied optional options, and
#        and optional options with defaults (possible overridden)
#
# Details: variable types can be "n" for numeric, 
#          "c" for character, "cv" for character vector, and "l" for logical
parseOptions = function(type, options, required, requiredTypes, 
                        optional, optionalTypes, lineNumOffset, defaults=NULL) {
  
  # Check that only valid options are included and that all required options are present
  optNames = names(options)
  optReq = c(required, optional)
  Sel = is.na(pmatch(optNames, optReq))
  if (sum(Sel)>0) {
    stop("@", lineNumOffset, " \\", type, " cannot include option(s): ", paste(optNames[Sel], collapse=", "),
         "\nValid options are ", paste(optReq, collapse=", "))
  }
  
  # Expand abbreviated option names to full names
  optNames = names(options) = optReq[pmatch(optNames, optReq)]
  
  # Assure all required options are present
  missingReq = setdiff(required, optNames)
  if (length(missingReq)>0) {
    stop("@", lineNumOffset, " \\", type, " requires option(s): ", paste(missingReq, collapse=", "))
  }
  
  # Construct vector of variable types matching 'options'
  nOpts = length(optNames)
  optTypes = rep(NA_character_, nOpts)
  optTypes[match(required, optNames)] = requiredTypes
  temp = match(optional, optNames)
  Sel = !is.na(temp)
  if (length(optTypes>0)) optTypes[match(optional[Sel], optNames)] = optionalTypes[Sel]
  assure(all(!is.na(optTypes)), 70026)
  
  #  Check data types of options and convert "cv" type to vectors
  for (opt in seq(along.with=optTypes)) {
    oType = optTypes[opt]
    if (oType == "n") {
      check = suppressWarnings(as.numeric(options[[opt]]))
      bad = is.na(check)
      if (any(bad)) {
        stop("@", lineNumOffset, " \\", type, " has non-numeric '", optNames[opt], "'")
      }
      options[[opt]] = check
    } else if (oType == "nv") {
      temp = strsplit(options[[opt]], ",")[[1]]
      check = sapply(temp, function(x) suppressWarnings(as.numeric(x)))
      bad = any(is.na(check))
      if (any(bad)) {
        stop("@", lineNumOffset, " \\", type, " has non-numeric '", optNames[opt], "'")
      }
      options[[opt]] = temp
    } else if (oType == "l") {
      check = suppressWarnings(as.logical(options[[opt]]))
      bad = is.na(check)
      if (any(bad)) {
        stop("@", lineNumOffset, " \\", type, " has non-logical '", optNames[opt], "'")
      }
      options[[opt]] = check
    } else if (oType == "cv") {
      options[[opt]] = sapply(strsplit(options[[opt]], ",")[[1]], str_trim)
    } else if (oType == "c") {
    } else {
      assure(FALSE, 221983)
    }
  } # end for each supplied option
  
  # Add in defaults
  neededDefaults = setdiff(names(defaults), optNames)
  if (!is.null(neededDefaults)) options = c(options, defaults[neededDefaults])
  
  return(options)
} # end parseOptions()



